//this file is part of eMule
//Copyright (C)2002 Merkur ( merkur-@users.sourceforge.net / http://www.emule-project.net )
//
//This program is free software; you can redistribute it and/or
//modify it under the terms of the GNU General Public License
//as published by the Free Software Foundation; either
//version 2 of the License, or (at your option) any later version.
//
//This program is distributed in the hope that it will be useful,
//but WITHOUT ANY WARRANTY; without even the implied warranty of
//MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//GNU General Public License for more details.
//
//You should have received a copy of the GNU General Public License
//along with this program; if not, write to the Free Software
//Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

#pragma once
#define CPING_USE_ICMP

namespace WombatAgent 
{
	//#ifdef CPING_USE_ICMP

	//These defines & structure definitions are taken from the "ipexport.h" and
	//"icmpapi.h" header files as provided with the Platform SDK and
	//are used internally by the CPing class. Including them here allows
	//you to compile the CPing code without the need to have the full 
	//Platform SDK installed.

	typedef unsigned long IPAddr;     // An IP address.

	typedef struct tagIP_OPTION_INFORMATION 
	{
		unsigned char      Ttl;              // Time To Live
		unsigned char      Tos;              // Type Of Service
		unsigned char      Flags;            // IP header flags
		unsigned char      OptionsSize;      // Size in bytes of options data
		unsigned char FAR *OptionsData;      // Pointer to options data
	} IP_OPTION_INFORMATION;

	typedef struct tagICMP_ECHOREPLY 
	{
		IPAddr                Address;       // Replying address
		unsigned long         Status;        // Reply IP_STATUS
		unsigned long         RoundTripTime; // RTT in milliseconds
		unsigned short        DataSize;      // Reply data size in bytes
		unsigned short        Reserved;      // Reserved for system use
		void FAR              *Data;         // Pointer to the reply data
		IP_OPTION_INFORMATION Options;       // Reply options
	} ICMP_ECHOREPLY;

	typedef IP_OPTION_INFORMATION FAR* LPIP_OPTION_INFORMATION;
	typedef ICMP_ECHOREPLY FAR* LPICMP_ECHOREPLY;
	typedef HANDLE (WINAPI IcmpCreateFile)(VOID);
	typedef IcmpCreateFile* lpIcmpCreateFile;
	typedef BOOL (WINAPI IcmpCloseHandle)(HANDLE IcmpHandle);
	typedef IcmpCloseHandle* lpIcmpCloseHandle;
	typedef DWORD (WINAPI IcmpSendEcho)(HANDLE IcmpHandle, IPAddr DestinationAddress,
		LPVOID RequestData, WORD RequestSize,
		LPIP_OPTION_INFORMATION RequestOptions,
		LPVOID ReplyBuffer, DWORD ReplySize, DWORD Timeout);
	typedef IcmpSendEcho* lpIcmpSendEcho;

	//#endif //CPING_USE_ICMP






	/////////////////////////// Classes /////////////////////////////////


	struct CPingReply
	{
		in_addr	 Address;  //The IP address of the replier
		unsigned long RTT; //Round Trip time in Milliseconds
		in_addr	 ip;
	};

	class CPing
	{
	public:
		//Methods
		//#ifdef CPING_USE_ICMP
		BOOL Ping1(LPCTSTR pszHostName, CPingReply& pr, UCHAR nTTL = 10, DWORD dwTimeout = 5000, UCHAR nPacketSize = 32) const;
		//#endif
		//#ifdef CPING_USE_WINSOCK2
		BOOL Ping2(LPCTSTR pszHostName, CPingReply& pr, UCHAR nTTL = 10, DWORD dwTimeout = 5000, UCHAR nPacketSize = 32) const;
		//#endif
	protected:
		//#ifdef CPING_USE_ICMP
		BOOL Initialise1() const;
		static BOOL sm_bAttemptedIcmpInitialise;
		static lpIcmpCreateFile sm_pIcmpCreateFile;
		static lpIcmpSendEcho sm_pIcmpSendEcho;
		static lpIcmpCloseHandle sm_pIcmpCloseHandle;
		//#endif

		//#ifdef CPING_USE_WINSOCK2
		BOOL Initialise2() const;
		static BOOL sm_bAttemptedWinsock2Initialise;
		static BOOL sm_bWinsock2OK;
		//#endif
		static BOOL IsSocketReadible(SOCKET socket, DWORD dwTimeout, BOOL& bReadible);
		static __int64 sm_TimerFrequency;
	};
}
#include <windows.h>
//#include <winsock.h> //<<< eWombat [WINSOCK2]
#include "types.h"
#include "Pinger.h"
// ZZ:UploadSpeedSense -->
#include "LastCommonRouteFinder.h"
// ZZ:UploadSpeedSense <--


typedef struct {
	float avg_ping;
	bool changed;
	float lowest_ping;
	bool lowest_changed;
	bool success;
} pingVal;

class CPingThread: public CWinThread
{
	DECLARE_DYNCREATE(CPingThread)

protected:
	CPingThread(void);

public:
	~CPingThread(void);
	virtual int	 Run();
	float	 Ping();
	pingVal  GetPing();
	bool	 isWorking()	{return works;}
	bool	 isPreparing (bool onlyLowestPing = false)  {return (prepares || !onlyLowestPing && searching);}
	bool SetIP(CString pingThreadNic);
	CString	 GetPingedServer();
	// ZZ:UploadSpeedSense -->
	bool	AddHostsToCheck(CTypedPtrList<CPtrList, CServer*> &list)	{return lastCommonRouteFinder->AddHostsToCheck(list);}
    bool	AddHostsToCheck(CTypedPtrList<CPtrList, CUpDownClient*> &list)	{return lastCommonRouteFinder->AddHostsToCheck(list);}
	// ZZ:UploadSpeedSense <--
	void StopPingThread(void);
	void StartPingThread(void);
	void EndPingThread(void);
	void		 Init();
	virtual	BOOL InitInstance() { return true; }

private:
	static const unsigned int m_lowest_ping_values = 3;
	void	findHostToPing();
	void	findLowestPing(bool useURL = false);
	// ZZ:UploadSpeedSense -->
    LastCommonRouteFinder*    lastCommonRouteFinder;
	Pinger*	pinger;
    PingStatus pingStatus;
	// ZZ:UploadSpeedSense <--

	CEvent*		endPingThread;

	float		avg_ping;		// Average ping
	bool		isPingThread;			// is pingThread enabled ?
	bool		works;
	bool		changed;
	bool		lowest_changed;

 	float		lowest_list[m_lowest_ping_values];
	int			lowest_index;
	float		lowest_ping;		// Lowest ping

	LPHOSTENT		pHost;			// Pointer to host entry structure
    struct in_addr	iaDest;			// Internet address structure
	DWORD			dwAddress;		// IP Address
	bool			findHost;
	bool			lowestPingIsFound;
	bool			prepares;
	bool			searching;
	bool			autoHost;
	bool			UDPing;
	bool			noTTL;
	bool			noTTLExec;
	uint32			lastCommonHost;
	uint32			lastCommonTTL;
	uint32			hostToPing;
	uint32			hostToPingNoTTL;
	uint8			failedTimes;
	uint32			lastRestartTime;
	uint16			timeOut;
};
